#ifndef qcolorutils_h
#define qcolorutils_h

/*
 * Copyright (C) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
 *
 * copied from kdelibs/kdeui/color/kcolorutils.h
 * Copyright (C) 2007 Matthew Woehlke <mw_triad@users.sourceforge.net>
 * Copyright (C) 2007 Thomas Zander <zander@kde.org>
 * Copyright (C) 2007 Zack Rusin <zack@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <QColor>

/*
 * A set of methods used to work with colors.
 */
namespace QColorUtils
{

//! set contrast values
void setContrast(qreal);

//! contrast
const qreal& contrast(void);

//! background contrast
const qreal& backgroundContrast(void);

//!@name color utilities
//@{
bool lowThreshold(const QColor&);
bool highThreshold(const QColor&);
QColor backgroundTopColor(const QColor&);
QColor backgroundBottomColor(const QColor&);
QColor backgroundRadialColor(const QColor&);

//! returns menu background color matching position in a top level widget of given height
QColor backgroundColor(const QColor &color, int height, int y);

//@}

/*!
Calculate the luma of a color. Luma is weighted sum of gamma-adjusted
R'G'B' components of a color. The result is similar to qGray. The range
is from 0.0 (black) to 1.0 (white).
*/
qreal luma(const QColor&);

/*!
Calculate the contrast ratio between two colors, according to the
W3C/WCAG2.0 algorithm, (Lmax + 0.05)/(Lmin + 0.05), where Lmax and Lmin
are the luma values of the lighter color and the darker color,
respectively.
*/
qreal contrastRatio(const QColor&, const QColor&);

//! Adjust the luma of a color by changing its distance from white.
QColor lighten(const QColor&, qreal amount = 0.5, qreal chromaInverseGain = 1.0);

//! Adjust the luma of a color by changing its distance from black.
QColor darken(const QColor&, qreal amount = 0.5, qreal chromaGain = 1.0);

//! Create a new color by tinting one color with another
QColor tint(const QColor &base, const QColor &color, qreal amount = 0.3);

//! mix two colors
QColor mix(const QColor &c1, const QColor &c2, qreal bias = 0.5);

enum ShadeRole {

    // The light color is lighter than dark() or shadow() and contrasts with the base color.
    LightShade,

    // The midlight color is in between base() and light().
    MidlightShade,

    // The mid color is in between base() and dark().
    MidShade,

    // The dark color is in between mid() and shadow().
    DarkShade,

    // The shadow color is darker than light() or midlight() and contrasts the base color.
    ShadowShade

};

QColor shade(const QColor&, ShadeRole);
QColor shade(const QColor&, ShadeRole, qreal contrast, qreal chromaAdjust = 0.0);
QColor shade(const QColor&, qreal lumaAmount, qreal chromaAmount = 0.0);
}

#endif
