/***************************************************************************
 *   Copyright (C) 2008 by David Sansome                                   *
 *   me@davidsansome.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "qt_rc_style.h"
#include "qt_style.h"
#include "wrapper.h"



static void      oxygenengine_rc_style_init(OxygenEngineRcStyle      *style);
static void      oxygenengine_rc_style_class_init(OxygenEngineRcStyleClass *klass);
static unsigned int     oxygenengine_rc_style_parse(GtkRcStyle          *rc_style,
        GtkSettings          *settings,
        GScanner             *scanner);
static void      oxygenengine_rc_style_merge(GtkRcStyle           *dest,
        GtkRcStyle           *src);

static GtkStyle *oxygenengine_rc_style_create_style(GtkRcStyle          *rc_style);

static GtkRcStyleClass *parent_class;

GType oxygenengine_type_rc_style = 0;


void oxygenengine_rc_style_register_type(GTypeModule *module)
{
    static const GTypeInfo object_info = {
        sizeof(OxygenEngineRcStyleClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) oxygenengine_rc_style_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof(OxygenEngineRcStyle),
        0,              /* n_preallocs */
        (GInstanceInitFunc) oxygenengine_rc_style_init,
        NULL
    };

    oxygenengine_type_rc_style = g_type_module_register_type(module, GTK_TYPE_RC_STYLE, "OxygenEngineRcStyle", &object_info, 0);
}

static void oxygenengine_rc_style_init(OxygenEngineRcStyle *style)
{
    (void) style;
}

static void oxygenengine_rc_style_class_init(OxygenEngineRcStyleClass *klass)
{
    GtkRcStyleClass *rc_style_class = GTK_RC_STYLE_CLASS(klass);

    parent_class = g_type_class_peek_parent(klass);

    rc_style_class->parse = oxygenengine_rc_style_parse;
    rc_style_class->merge = oxygenengine_rc_style_merge;
    rc_style_class->create_style = oxygenengine_rc_style_create_style;
}




static unsigned int
oxygenengine_rc_style_parse(GtkRcStyle *rc_style, GtkSettings *settings, GScanner *scanner)
{
    (void) rc_style;
    (void) settings;
    static GQuark       scope_id = 0;
    unsigned int               old_scope;
    unsigned int               token;

    /* The rest of this keeps GTK happy - therefore I don't care what it does */
    if(!scope_id)
        scope_id = g_quark_from_string("theme_engine");

    old_scope = g_scanner_set_scope(scanner, scope_id);

    token = g_scanner_get_next_token(scanner);
    while(token != G_TOKEN_RIGHT_CURLY) {
        token = g_scanner_get_next_token(scanner);
    }

    g_scanner_set_scope(scanner, old_scope);

    return G_TOKEN_NONE;
}

static void
oxygenengine_rc_style_merge(GtkRcStyle * dest,
                            GtkRcStyle * src)
{
    parent_class->merge(dest, src);
}

/* Create an empty style suitable to this RC style
 */
static GtkStyle *
oxygenengine_rc_style_create_style(GtkRcStyle *rc_style)
{
    (void) rc_style;
    void *ptr = GTK_STYLE(g_object_new(QTENGINE_TYPE_STYLE, NULL));
    return (GtkStyle *)ptr;
}
