/***************************************************************************
 *   Copyright (C) 2008 by David Sansome                                   *
 *   me@davidsansome.com                                                   *
 *                                                                         *
 *   Copyright (C) 2010 by Cédric Bellegarde                               *
 *   gnumdk@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtkprogressbar.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <glib/glist.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "qt_style.h"
#include "qt_rc_style.h"
#include "wrapper.h"
#include "wmmove.c" /* Code from qtcurve engine */

/* Special fix for Qt oxygen drawing extension when drawing tabs */
#define QT_GTK_TAB_DIFF 6
#define GTK_ACTIVE_TAB_FIX 2

#define DETAIL(xx)    ((detail) && (!strcmp(xx, detail)))
#define DETAILHAS(xx) ((detail) && (strstr(detail, xx)))
#define PARENT(xx)    ((parent) && (!strcmp(xx, gtk_widget_get_name(parent))))
#define WIDGET(xx)    (GTK_IS_WIDGET(widget) && (!strcmp(xx, gtk_widget_get_name(widget))))

#define GTK_QT_STANDARD_ARGS window, toplevel->window, style, stateType, x, y, w, h, GTK_IS_WIDGET(widget) && gtk_widget_is_focus(widget)

#ifndef max
#define max(x,y) ((x)>=(y)?(x):(y))
#endif
#ifndef min
#define min(x,y) ((x)<=(y)?(x):(y))
#endif

#define IS_MENU_WINDOW(Window) (GTK_WINDOW(Window)->default_widget && GTK_IS_MENU(GTK_WINDOW(Window)->default_widget))

static void oxygenengine_style_init(OxygenEngineStyle      *style);
static void oxygenengine_style_class_init(OxygenEngineStyleClass *klass);

static GtkStyleClass *parent_class = NULL;

static void windowCleanup(GtkWidget *widget)
{
    if(widget) {
        g_signal_handler_disconnect(G_OBJECT(widget),
                                    (unsigned long)g_object_steal_data(G_OBJECT(widget), "WINDOW_SIZE_REQUEST_ID"));
        g_signal_handler_disconnect(G_OBJECT(widget),
                                    (unsigned long)g_object_steal_data(G_OBJECT(widget), "WINDOW_DESTROY_ID"));
        g_signal_handler_disconnect(G_OBJECT(widget),
                                    (unsigned long)g_object_steal_data(G_OBJECT(widget), "WINDOW_STYLE_SET_ID"));
        g_object_steal_data(G_OBJECT(widget), "WINDOW_HACK_SET");
    }
}

static gboolean windowSizeRequest(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    (void) event;
    (void) user_data;
    /* Need to invalidate the whole of the window on a resize, as gradient needs to be redone.*/
    if(widget) {
        int width, height;
        /* Need to check size really change, some apps are stupid */
        sizeFromMap(GDK_DRAWABLE_XID(widget->window), &width, &height);
        if((width != -1 && width != widget->allocation.width) || (height != -1 && height != widget->allocation.height)) {
            GdkRectangle rect;
            rect.x = 0, rect.y = 0, rect.width = widget->allocation.width, rect.height = widget->allocation.height;
            gdk_window_invalidate_rect(widget->window, &rect, FALSE);
            addSize2Map(GDK_DRAWABLE_XID(widget->window), widget->allocation.width, widget->allocation.height);
            return TRUE;
        }
    }
    return FALSE;
}

static gboolean windowStyleSet(GtkWidget *widget, GtkStyle *previous_style, gpointer user_data)
{
    (void) previous_style;
    (void) user_data;
    windowCleanup(widget);
    return FALSE;
}

static gboolean windowDestroy(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    (void) event;
    (void) user_data;
    int width = 0;
    int height = 0;
    if(GDK_IS_DRAWABLE(widget->window))
        gdk_drawable_get_size(widget->window, &width, &height);
    destroyImage(GDK_WINDOW_XID(widget->window), width, height);
    windowCleanup(widget);
    return FALSE;
}

static gboolean windowSetup(GtkWidget *widget)
{
    if(widget && !g_object_get_data(G_OBJECT(widget), "WINDOW_HACK_SET")) {
        g_object_set_data(G_OBJECT(widget), "WINDOW_HACK_SET", (gpointer)1);

        g_object_set_data(G_OBJECT(widget), "WINDOW_SIZE_REQUEST_ID",
                          (gpointer)g_signal_connect(G_OBJECT(widget), "size-allocate",
                                  (GtkSignalFunc)windowSizeRequest, NULL));
        g_object_set_data(G_OBJECT(widget), "WINDOW_DESTROY_ID",
                          (gpointer)g_signal_connect(G_OBJECT(widget), "hide",
                                  (GtkSignalFunc)windowDestroy, NULL));
        g_object_set_data(G_OBJECT(widget), "WINDOW_STYLE_SET_ID",
                          (gpointer)g_signal_connect(G_OBJECT(widget), "style-set",
                                  (GtkSignalFunc)windowStyleSet, NULL));
        return TRUE;
    }

    return FALSE;
}

static void sanitize_size(GdkWindow* window, int* w, int* h)
{
    if((*w == -1) && (*h == -1))
        gdk_window_get_size(window, w, h);
    else if(*w == -1)
        gdk_window_get_size(window, w, NULL);
    else if(*h == -1)
        gdk_window_get_size(window, NULL, h);
}

static void
draw_hline(GtkStyle* style,
           GdkWindow* window,
           GtkStateType stateType,
           GdkRectangle* area,
           GtkWidget* widget,
           const gchar* detail,
           int x1,
           int x2,
           int y)
{
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    int x = min(x1, x2);
    int w = abs(x2 - x1);
    int h = 2;

    if(gtkQtDebug())
        printf("HLINE (%d,%d,%d) Widget: %s  Detail: %s\n", x1, x2, y, gtk_widget_get_name(widget), detail);

    if(DETAIL("menuitem") || ! DETAIL("vscale"))
        drawToolMenuItem(GTK_QT_STANDARD_ARGS, 2, FALSE);
}


static void
draw_vline(GtkStyle* style,
           GdkWindow* window,
           GtkStateType stateType,
           GdkRectangle* area,
           GtkWidget* widget,
           const gchar* detail,
           int y1,
           int y2,
           int x)
{
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    int y = min(y1, y2) - 5;
    int h = abs(y2 - y1) + 10;
    int w = 2;

    if(gtkQtDebug())
        printf("VLINE (%d,%d,%d) Widget: %s  Detail: %s\n", y1, y2, x, gtk_widget_get_name(widget), detail);

    if(DETAIL("toolbar") || !DETAIL("hscale")) {
        drawToolMenuItem(GTK_QT_STANDARD_ARGS, 2, TRUE);
    }
}

static void
draw_shadow(GtkStyle     *style,
            GdkWindow    *window,
            GtkStateType  stateType,
            GtkShadowType shadowType,
            GdkRectangle *area,
            GtkWidget    *widget,
            const gchar  *detail,
            int          x,
            int          y,
            int          w,
            int          h)
{
    (void) area;
    GtkWidget *toplevel = NULL;
    int tx, ty;

    sanitize_size(window, &w, &h);

    if(gtkQtDebug())
        printf("Shadow (%d,%d,%d,%d) Widget: %s Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    if(!widget)  /* stupid gftp */
        return;

    toplevel = gtk_widget_get_toplevel(widget);

    if(DETAIL("menuitem"))
        return;
    if(DETAIL("menu"))
        return;
    if(DETAIL("scrolled_window")) {
        int tx, ty;
        if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
            x -= 1;
            y -= 0;
            w += 2;
            h += 2;
            drawRectFrame(GTK_QT_STANDARD_ARGS, tx, ty);
        } else
            drawMenuFrame(GTK_QT_STANDARD_ARGS);
        return;
    }
    if(DETAIL("entry")) {
        if(gtk_widget_has_focus(widget))
            stateType = GTK_STATE_SELECTED;
        if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
            GtkWidget *parent = gtk_widget_get_parent(widget);
            /* Here we check if current window is a mozilla (GtkFixed) window,
             * we already have this information with noGradients() but we need
             * to handle gtk open/save dialogs
             */
            drawLineEditFrame(GTK_QT_STANDARD_ARGS, tx, ty, !strcmp(gtk_widget_get_name(parent), "GtkFixed"));
        }
        return;
    }
    if(DETAIL("frame") || DETAIL("trough") || DETAIL("viewport")) {
        return;
    }

    if(shadowType == GTK_SHADOW_NONE)
        return;

    if(WIDGET("GimpFgBgEditor"))
        return;
    /*
    if (gtk_widget_translate_coordinates (widget, toplevel, x, y, &tx, &ty))
    {
        drawFrame(GTK_QT_STANDARD_ARGS, 1, tx, ty, GTK_POS_TOP);
    }*/
}

static void
draw_polygon(GtkStyle* style,
             GdkWindow* window,
             GtkStateType stateType,
             GtkShadowType shadowType,
             GdkRectangle* area,
             GtkWidget* widget,
             const gchar* detail,
             GdkPoint* points,
             int npoints,
             int fill)
{
#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif /* M_PI */
#ifndef M_PI_4
#define M_PI_4  0.78539816339744830962
#endif /* M_PI_4 */
    (void) widget;
    (void) detail;
    (void) shadowType;
    static const gdouble pi_over_4 = M_PI_4;
    static const gdouble pi_3_over_4 = M_PI_4 * 3;

    GdkGC              *gc1;
    GdkGC              *gc2;
    GdkGC              *gc3;
    GdkGC              *gc4;
    gdouble             angle;
    int                xadjust;
    int                yadjust;
    int                i;

    g_return_if_fail(style != NULL);
    g_return_if_fail(window != NULL);
    g_return_if_fail(points != NULL);

    switch(shadowType) {
    case GTK_SHADOW_IN:
        gc1 = style->light_gc[stateType];
        gc2 = style->dark_gc[stateType];
        gc3 = style->light_gc[stateType];
        gc4 = style->dark_gc[stateType];
        break;
    case GTK_SHADOW_ETCHED_IN:
        gc1 = style->light_gc[stateType];
        gc2 = style->dark_gc[stateType];
        gc3 = style->dark_gc[stateType];
        gc4 = style->light_gc[stateType];
        break;
    case GTK_SHADOW_OUT:
        gc1 = style->dark_gc[stateType];
        gc2 = style->light_gc[stateType];
        gc3 = style->dark_gc[stateType];
        gc4 = style->light_gc[stateType];
        break;
    case GTK_SHADOW_ETCHED_OUT:
        gc1 = style->dark_gc[stateType];
        gc2 = style->light_gc[stateType];
        gc3 = style->light_gc[stateType];
        gc4 = style->dark_gc[stateType];
        break;
    default:
        return;
    }

    if(area) {
        gdk_gc_set_clip_rectangle(gc1, area);
        gdk_gc_set_clip_rectangle(gc2, area);
        gdk_gc_set_clip_rectangle(gc3, area);
        gdk_gc_set_clip_rectangle(gc4, area);
    }

    if(fill)
        gdk_draw_polygon(window, style->bg_gc[stateType], TRUE, points, npoints);

    npoints--;

    for(i = 0; i < npoints; i++) {
        if((points[i].x == points[i + 1].x) &&
                (points[i].y == points[i + 1].y)) {
            angle = 0;
        } else {
            angle = atan2(points[i + 1].y - points[i].y,
                          points[i + 1].x - points[i].x);
        }

        if((angle > -pi_3_over_4) && (angle < pi_over_4)) {
            if(angle > -pi_over_4) {
                xadjust = 0;
                yadjust = 1;
            } else {
                xadjust = 1;
                yadjust = 0;
            }

            gdk_draw_line(window, gc1,
                          points[i].x - xadjust, points[i].y - yadjust,
                          points[i + 1].x - xadjust, points[i + 1].y - yadjust);
            gdk_draw_line(window, gc3,
                          points[i].x, points[i].y,
                          points[i + 1].x, points[i + 1].y);
        } else {
            if((angle < -pi_3_over_4) || (angle > pi_3_over_4)) {
                xadjust = 0;
                yadjust = 1;
            } else {
                xadjust = 1;
                yadjust = 0;
            }

            gdk_draw_line(window, gc4,
                          points[i].x + xadjust, points[i].y + yadjust,
                          points[i + 1].x + xadjust, points[i + 1].y + yadjust);
            gdk_draw_line(window, gc2,
                          points[i].x, points[i].y,
                          points[i + 1].x, points[i + 1].y);
        }
    }
    if(area) {
        gdk_gc_set_clip_rectangle(gc1, NULL);
        gdk_gc_set_clip_rectangle(gc2, NULL);
        gdk_gc_set_clip_rectangle(gc3, NULL);
        gdk_gc_set_clip_rectangle(gc4, NULL);
    }
}

static void
draw_arrow(GtkStyle* style,
           GdkWindow* window,
           GtkStateType stateType,
           GtkShadowType shadowType,
           GdkRectangle* area,
           GtkWidget* widget,
           const gchar *detail,
           GtkArrowType arrowType,
           int fill, int x, int y, int w, int h)
{
    (void) area;
    (void) shadowType;
    (void) fill;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    sanitize_size(window, &w, &h);

    if(gtkQtDebug())
        printf("Arrow (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    if(DETAIL("menuitem") && !noGradients()) {
        if(stateType == GTK_STATE_PRELIGHT) {/* arraw already drawn by Qt */ 
            return;
        }
        else {
            switch(arrowType) {
                case GTK_ARROW_LEFT:
                    x -= 2;
                    break;
                case GTK_ARROW_RIGHT:
                    x += 2;
                    break;
                default:
                    break;
            }
        }
    }
    
    if(DETAIL("hscrollbar") || DETAIL("vscrollbar"))
        return;
    if(DETAIL("spinbutton"))
        return;
    if(DETAIL("notebook")) {
        drawArrow(GTK_QT_STANDARD_ARGS, arrowType);
        return;
    }
    if(DETAIL("arrow")) {
        GtkWidget* parent = gtk_widget_get_parent(widget);
        if(parent) {
            drawArrow(GTK_QT_STANDARD_ARGS, arrowType);
        }
        return;
    } else {
        drawArrow(GTK_QT_STANDARD_ARGS, arrowType);
        return;
    }
}



static void
draw_diamond(GtkStyle * style,
             GdkWindow * window,
             GtkStateType stateType,
             GtkShadowType shadowType,
             GdkRectangle * area,
             GtkWidget * widget,
             const gchar *detail,
             int x,
             int y,
             int w,
             int h)
{
    (void) style;
    (void) window;
    (void) stateType;
    (void) shadowType;
    (void) area;
    (void) widget;
    (void) detail;
    (void) x;
    (void) y;
    (void) w;
    (void) h;
}



static void
draw_box(GtkStyle * style,
         GdkWindow * window,
         GtkStateType stateType,
         GtkShadowType shadowType,
         GdkRectangle * area,
         GtkWidget * widget,
         const gchar *detail,
         int x,
         int y,
         int w,
         int h)
{
    (void) shadowType;
    (void) area;
    int tx, ty;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);

    sanitize_size(window, &w, &h);

    if(gtkQtDebug())
        printf("Box (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    if(GTK_IS_SCROLLBAR(widget)) {
        if(DETAIL("trough")) {
            GtkOrientation orientation = ((w > h) ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);

            drawScrollBar(GTK_QT_STANDARD_ARGS, orientation);
        }
        return;
    }
    if(DETAIL("menuitem")) {
        /* Crude way of checking if it's a menu item, or a menubar item */
        if(x != 0)
            drawMenuBarItem(GTK_QT_STANDARD_ARGS);
        else {
            int type = 0;
            if(GTK_IS_TEAROFF_MENU_ITEM(widget))
                return;
            else if (!noGradients() && gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget)))
                type = 3;
            drawToolMenuItem(GTK_QT_STANDARD_ARGS, type, FALSE);
        }
        return;
    }
    if(DETAIL("menubar")) {
        if(!noGradients()) {
            int tx, ty;
            if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
                addSize2Map(GDK_DRAWABLE_XID(widget->window), widget->allocation.width, widget->allocation.height);
                windowSetup(widget);
                qtcWMMoveSetup(widget);
                drawWidgetBackground(GTK_QT_STANDARD_ARGS, tx, ty);
            }
        }
        return;
    }
    if(DETAIL("menu")) {
        x -= 1;
        y -= 1;
        w += 2;
        h += 3;
        drawMenuFrame(GTK_QT_STANDARD_ARGS);
        return;
    }
    if(GTK_IS_PROGRESS(widget) && DETAIL("trough")) {
        double fraction = gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(widget));
        GtkProgressBarOrientation orientation = gtk_progress_bar_get_orientation(GTK_PROGRESS_BAR(widget));
        drawProgressBar(GTK_QT_STANDARD_ARGS, orientation, fraction);
        return;
    }
    if(DETAIL("bar")) {
        if((GTK_IS_PROGRESS(widget) && !gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(widget))) || noGradients()) {
            x = area->x; y = area->y; w = area->width; h = area->height;
            drawProgressChunk(GTK_QT_STANDARD_ARGS);
        }
        return;
    }
    if(GTK_IS_SCALE(widget) && DETAIL("trough")) {
        GtkAdjustment* adj;
        int inverted;
        GValue *val = (GValue*)g_malloc(sizeof(GValue));

        memset(val, 0, sizeof(GValue));
        g_value_init(val, G_TYPE_BOOLEAN);
        g_object_get_property((GObject*)widget, "inverted", val);
        inverted = g_value_get_boolean(val);
        g_value_unset(val);
        g_free(val);

        adj = gtk_range_get_adjustment((GtkRange *) widget);
        drawSlider(GTK_QT_STANDARD_ARGS, adj, (GTK_RANGE(widget))->orientation, inverted);
        return;
    }
    if(DETAIL("button")) {
        GtkWidget *parent;
        GtkReliefStyle relief;
        parent = gtk_widget_get_parent(widget);

        if(parent) {
            if(GTK_IS_CLIST(parent) || GTK_IS_LIST(parent) || GTK_IS_TREE_VIEW(parent)) {
                drawListHeader(GTK_QT_STANDARD_ARGS);
                return;
            }
        }
        if(GTK_IS_BUTTON(widget)) {
            relief = gtk_button_get_relief(GTK_BUTTON(widget));
        }
        else {
            stateType = GTK_STATE_NORMAL;
            drawMenuFrame(GTK_QT_STANDARD_ARGS);
            return;
        }
        
        if(relief == GTK_RELIEF_NONE || relief == GTK_RELIEF_HALF) {
            drawToolButton(GTK_QT_STANDARD_ARGS);
        } else { /* normal button */
            int defaultButton = GTK_WIDGET_HAS_FOCUS(widget);
            GtkWindow *top_window = GTK_WINDOW(toplevel);

            if(top_window && top_window->default_widget == widget)
                defaultButton = 1;

            drawButton(GTK_QT_STANDARD_ARGS, defaultButton);
        }
        return;
    }
    if(DETAIL("tab")) {
        GtkWidget *toplevel = gtk_widget_get_toplevel(widget);
        /* Gtk doesn't draw tab at same qt position,
        */
        h += QT_GTK_TAB_DIFF;
        drawTab(GTK_QT_STANDARD_ARGS, GTK_POS_TOP);

        return;
    }
    if(DETAIL("optionmenu")) {
        drawComboBox(GTK_QT_STANDARD_ARGS);
        return;
    }
    if(DETAIL("toolbar")) { /* Nothing to draw for now*/
        return;
    }
    if(DETAIL("spinbutton_up")) {
        if(noGradients())
            drawSpinButton(GTK_QT_STANDARD_ARGS, 0);
        else {
            x -= 2;
            y += 2;
            drawArrow(GTK_QT_STANDARD_ARGS, GTK_ARROW_UP);
        }
        return;
    }
    if(DETAIL("spinbutton_down")) {
        if(noGradients())
            drawSpinButton(GTK_QT_STANDARD_ARGS, 1);
        else {
            x -= 2;
            drawArrow(GTK_QT_STANDARD_ARGS, GTK_ARROW_DOWN);
        }
        return;
    }
    if(DETAIL("spinbutton")) {
        if(!noGradients()) {
            if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
                drawSpinFrame(GTK_QT_STANDARD_ARGS, tx, ty);
            }
        }
        return;
    }

    if(DETAIL("optionmenutab") || DETAIL("buttondefault") || DETAIL("trough-fill-level"))
        return;
    
    if(DETAIL("entry-progress")) {
        drawProgressChunk(GTK_QT_STANDARD_ARGS);
        return;
    }
    
    if(!detail) {
        parent_class->draw_box(style, window, stateType, shadowType, area, widget, detail, x, y, w, h);
        return;
    }
    
    if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
        drawFrame(GTK_QT_STANDARD_ARGS, 0, tx, ty, GTK_POS_TOP);
    }
}

/* From qtcurve, see at end of draw_flat_box() */
static void qtcLogHandler(const gchar *domain, GLogLevelFlags level, const gchar *msg, gpointer data)
{
    (void) domain;
    (void) level;
    (void) msg;
    (void) data;
}

static void
draw_flat_box(GtkStyle * style,
              GdkWindow * window,
              GtkStateType stateType,
              GtkShadowType shadowType,
              GdkRectangle * area,
              GtkWidget * widget,
              const gchar *detail,
              int x,
              int y,
              int w,
              int h)
{
    (void) shadowType;
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    sanitize_size(window, &w, &h);

    if(gtkQtDebug())
        printf("Flat Box (%d,%d,%d,%d) Widget: %s  Detail: %s %d %d\n", x, y, w, h, gtk_widget_get_name(widget), detail, stateType, GTK_STATE_SELECTED);

    if((DETAILHAS("cell_odd_ruled")|| DETAIL("treeview")) && stateType != GTK_STATE_SELECTED) {
        cairo_t *t = gdk_cairo_create(window);
        GdkColor color = alternateBackground(style, stateType != GTK_STATE_INSENSITIVE);
        cairo_set_source_rgb(t, (float)color.red/65535, (float)color.green/65535, (float)color.blue/65535);
        cairo_rectangle(t, x, y, w, h);
        cairo_fill(t);
        cairo_destroy(t);
    } else if((DETAILHAS("cell_odd") || DETAILHAS("cell_even") || DETAIL("listitem")) && (stateType == GTK_STATE_SELECTED)) {
        unsigned int round;
        if(strstr(detail, "_start") != 0)
            round = 0;
        else if(strstr(detail, "_middle") != 0)
            round = 1;
        else if(strstr(detail, "_end") != 0)
            round = 2;
        else
            round = 3;
        drawItemViewItem(GTK_QT_STANDARD_ARGS, round);
    } else if(DETAIL("tooltip")) {
        gtk_window_set_opacity(GTK_WINDOW(widget), 0.8);
        drawTooltip(GTK_QT_STANDARD_ARGS);
    } else if(DETAIL("entry_bg")) {
        gboolean editable = TRUE;
        if(!noGradients() && GTK_IS_ENTRY(widget)) {
            if(!gtk_widget_is_sensitive(widget))
                editable = FALSE;
            drawLineEdit(GTK_QT_STANDARD_ARGS);
        }
    } else if(DETAIL("trough")) {
        int tx, ty;
        if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
            int topWidth = 0;
            int topHeight =0;
            if(GDK_IS_DRAWABLE(toplevel->window))
                gdk_drawable_get_size(toplevel->window, &topWidth, &topHeight);
            if(topWidth >= tx + w && topHeight >= ty + h)
                drawWidgetBackground(GTK_QT_STANDARD_ARGS, tx, ty);
            else
                drawRect(GTK_QT_STANDARD_ARGS, TRUE);
        }
    } else if(DETAIL("viewportbin")) {
        int tx, ty;
        if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
            drawViewportBackground(GTK_QT_STANDARD_ARGS, tx, ty);
        }
    } else if(DETAIL("eventbox")) {
        int tx, ty;
        if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
            drawWidgetBackground(GTK_QT_STANDARD_ARGS, tx, ty);
        }
    } else if(DETAIL("icon_view_item")) {
        drawItemViewItem(GTK_QT_STANDARD_ARGS, 3);
    } else if(DETAIL("base")) {
        if(GTK_IS_WINDOW(widget) &&
                gtk_widget_is_toplevel(widget) &&
                ! IS_MENU_WINDOW(widget) &&
                ! noGradients()) {
            addSize2Map(GDK_DRAWABLE_XID(widget->window), widget->allocation.width, widget->allocation.height);
            windowSetup(widget);
            drawOxygenBackground(GTK_QT_STANDARD_ARGS);
        }
        /* Code from qtcurve */
        if(GTK_IS_DIALOG(toplevel) && !g_object_get_data(G_OBJECT(toplevel), "QTC_BUTTON_ORDER_HACK_SET")) {
            /* gtk_dialog_set_alternative_button_order will cause errors to be logged, but dont want these
               so register ur own error handler, and then unregister afterwards...*/
            guint id=g_log_set_handler("Gtk", G_LOG_LEVEL_CRITICAL, qtcLogHandler, NULL);
            g_object_set_data(G_OBJECT(toplevel), "QTC_BUTTON_ORDER_HACK_SET", (gpointer)1);

            gtk_dialog_set_alternative_button_order(GTK_DIALOG(toplevel), GTK_RESPONSE_HELP,
                                                    GTK_RESPONSE_OK, GTK_RESPONSE_YES, GTK_RESPONSE_ACCEPT, GTK_RESPONSE_APPLY,
                                                    GTK_RESPONSE_REJECT, GTK_RESPONSE_CLOSE, GTK_RESPONSE_NO, GTK_RESPONSE_CANCEL, -1);
            g_log_remove_handler("Gtk", id);
            g_log_set_handler("Gtk", G_LOG_LEVEL_CRITICAL, g_log_default_handler, NULL);
        }
    }
}


static void
draw_check(GtkStyle * style,
           GdkWindow * window,
           GtkStateType stateType,
           GtkShadowType shadowType,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           int x,
           int y,
           int w,
           int h)
{
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    if(gtkQtDebug())
        printf("Check (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    if(GTK_IS_MENU_ITEM(widget)) {
        if(shadowType == GTK_SHADOW_IN) {
            drawMenuCheck(GTK_QT_STANDARD_ARGS);
        }
        return;
    }
    drawCheckBox(GTK_QT_STANDARD_ARGS, shadowType == GTK_SHADOW_IN);
}

static void
draw_option(GtkStyle * style,
            GdkWindow * window,
            GtkStateType stateType,
            GtkShadowType shadowType,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            int x,
            int y,
            int w,
            int h)
{
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    if(gtkQtDebug())
        printf("Option (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    if(GTK_IS_MENU_ITEM(widget)) {
        if(shadowType == GTK_SHADOW_IN) {
            drawMenuCheck(GTK_QT_STANDARD_ARGS);
        }
        return;
    }
    drawRadioButton(GTK_QT_STANDARD_ARGS, shadowType == GTK_SHADOW_IN);
}


static void
draw_tab(GtkStyle * style,
         GdkWindow * window,
         GtkStateType stateType,
         GtkShadowType shadowType,
         GdkRectangle * area,
         GtkWidget * widget,
         const gchar *detail,
         int x,
         int y,
         int w,
         int h)
{
    (void) shadowType;
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
    if(gtkQtDebug())
        printf("Tab (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    drawArrow(GTK_QT_STANDARD_ARGS, GTK_ARROW_DOWN);
}

static void
draw_box_gap(GtkStyle* style,
             GdkWindow* window,
             GtkStateType stateType,
             GtkShadowType shadowType,
             GdkRectangle* area,
             GtkWidget* widget,
             const gchar* detail,
             int x,
             int y,
             int w,
             int h,
             GtkPositionType gap_side,
             int gap_x,
             int gap_w)
{
    (void) shadowType;
    (void) area;
    (void) gap_side;
    int tx, ty;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);

    sanitize_size(window, &w, &h);

    if(w < 0 || h < 10) return; /* Eclipse/Firefox really can be this stupid! */

    if(gtkQtDebug())
        printf("Box_gap (%d,%d,%d,%d,%d, %d) Widget: %s  Detail: %s\n", x, y, w, h, gap_x, gap_w, gtk_widget_get_name(widget), detail);

    if(DETAIL("notebook")) {
        GtkPositionType position = gtk_notebook_get_tab_pos(GTK_NOTEBOOK(widget));
        if(position == GTK_POS_BOTTOM || position == GTK_POS_TOP) {
            x -= 2;
            w += 4;
        }
        else {
            y -= 2;
            h += 4;
        }
        if(gtk_widget_translate_coordinates(widget, toplevel, x, y, &tx, &ty)) {
            drawFrame(GTK_QT_STANDARD_ARGS, 0, tx, ty, position);
        }
    }
}

static void
draw_shadow_gap(GtkStyle * style,
                GdkWindow * window,
                GtkStateType stateType,
                GtkShadowType shadowType,
                GdkRectangle * area,
                GtkWidget * widget,
                const gchar *detail,
                int x,
                int y,
                int w,
                int h,
                GtkPositionType gap_side,
                int gap_x,
                int gap_w)
{
    (void) style;
    (void) window;
    (void) stateType;
    (void) shadowType;
    (void) area;
    (void) widget;
    (void) detail;
    (void) x;
    (void) y;
    (void) w;
    (void) h;
    (void) gap_side;
    (void) gap_x;
    (void) gap_w;
}

static void
draw_extension(GtkStyle * style,
               GdkWindow * window,
               GtkStateType stateType,
               GtkShadowType shadowType,
               GdkRectangle * area,
               GtkWidget * widget,
               const gchar *detail,
               int x,
               int y,
               int w,
               int h,
               GtkPositionType gap_side)
{
    (void) gap_side;
    (void) shadowType;
    (void) area;
    GtkWidget *toplevel;
    GtkPositionType position;

    g_return_if_fail(style != NULL);
    g_return_if_fail(window != NULL);

    sanitize_size(window, &w, &h);

    if(gtkQtDebug())
        printf("Extension (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    toplevel = gtk_widget_get_toplevel(widget);
    position = gtk_notebook_get_tab_pos(GTK_NOTEBOOK(widget));
    
    /* Gtk doesn't draw tab at same qt position,
     * This try to handle the problem
     */
    switch(position) {
    case GTK_POS_LEFT:
        break;
    case GTK_POS_RIGHT:
        break;
    default:
        h += QT_GTK_TAB_DIFF;
        if(stateType == GTK_STATE_ACTIVE) {
            y -= GTK_ACTIVE_TAB_FIX;
            h += GTK_ACTIVE_TAB_FIX;
        }
        break;
    }

    drawTab(GTK_QT_STANDARD_ARGS, position);
}


static void
draw_focus(GtkStyle     *style,
           GdkWindow    *window,
           GtkStateType  stateType,
           GdkRectangle *area,
           GtkWidget    *widget,
           const gchar  *detail,
           int          x,
           int          y,
           int          w,
           int          h)
{
    (void) style;
    (void) window;
    (void) stateType;
    (void) area;
    (void) widget;
    (void) detail;
    (void) x;
    (void) y;
    (void) w;
    (void) h;
}

static void
draw_slider(GtkStyle * style,
            GdkWindow * window,
            GtkStateType stateType,
            GtkShadowType shadowType,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            int x,
            int y,
            int w,
            int h,
            GtkOrientation orientation)
{
    (void) shadowType;
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);

    if(gtkQtDebug())
        printf("Slider (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, w, h, gtk_widget_get_name(widget), detail);

    if(DETAIL("slider")) {
        drawScrollBarSlider(GTK_QT_STANDARD_ARGS, orientation);
        return;
    }
}

static void
draw_handle(GtkStyle * style,
            GdkWindow * window,
            GtkStateType stateType,
            GtkShadowType shadowType,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            int x,
            int y,
            int w,
            int h,
            GtkOrientation orientation)
{
    (void) shadowType;
    (void) area;
    GtkWidget* toplevel = gtk_widget_get_toplevel(widget);

    g_return_if_fail(style != NULL);
    g_return_if_fail(window != NULL);

    sanitize_size(window, &w, &h);

    if(gtkQtDebug())
        printf("Handle (%d,%d,%d,%d) Widget: %s  Detail: %s \n", x, y, w, h, gtk_widget_get_name(widget), detail);

    drawSplitter(GTK_QT_STANDARD_ARGS, orientation);
    return;
}

static
void  draw_layout(GtkStyle *style,
                  GdkWindow *window,
                  GtkStateType stateType,
                  gboolean use_text,
                  GdkRectangle *area,
                  GtkWidget *widget,
                  const gchar *detail,
                  int x,
                  int y,
                  PangoLayout *layout)
{
    (void) use_text;
    cairo_t *t = gdk_cairo_create(window);
    GdkColor color;
    float opacity = 1.0;
    
    if(gtkQtDebug())
        printf("Layout (%d,%d) Widget: %s, Layout: %s,  Detail: %s %d \n", x, y, gtk_widget_get_name(widget), pango_layout_get_text(layout), detail, stateType);

    g_return_if_fail(window != NULL);
    
    if(area) {
        gdk_cairo_rectangle(t, area);
        cairo_clip(t);
    }
    
    if(stateType == GTK_STATE_INSENSITIVE)
        opacity = 0.45;
    
    switch(stateType) {
        case GTK_STATE_PRELIGHT: /* Not Kde style */
            color = style->text[GTK_STATE_NORMAL];
            break;
        case GTK_STATE_ACTIVE:
            if(DETAIL("cellrenderertext")) /* Only in item list */
                color = style->text[stateType];
            else
                color = style->text[GTK_STATE_NORMAL];
            break;
        case GTK_STATE_INSENSITIVE: /* For a strange reason, GTK_STATE_INSENSITIVE color is bad */
            color = style->text[GTK_STATE_NORMAL];
            opacity = 0.45;
            break;
        default:
            color = style->text[stateType];
            break;
    }
 
    cairo_set_source_rgba(t, (float)color.red/65535, (float)color.green/65535, (float)color.blue/65535, opacity);
    cairo_move_to(t, x, y);
    pango_cairo_show_layout(t, layout);
    cairo_destroy(t);

}

GType oxygenengine_type_style = 0;

void
oxygenengine_style_register_type(GTypeModule *module)
{
    static const GTypeInfo object_info = {
        sizeof(OxygenEngineStyleClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) oxygenengine_style_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof(OxygenEngineStyle),
        0,              /* n_preallocs */
        (GInstanceInitFunc) oxygenengine_style_init,
        NULL
    };

    oxygenengine_type_style = g_type_module_register_type(module,
                              GTK_TYPE_STYLE,
                              "OxygenEngineStyle",
                              &object_info, 0);
}

static void
oxygenengine_style_init(OxygenEngineStyle *style)
{
    (void) style;
}

static GdkPixbuf *
set_transparency(const GdkPixbuf *pixbuf, gdouble alpha_percent)
{
    GdkPixbuf *target;
    guchar *data, *current;
    unsigned int x, y, rowstride, height, width;

    g_return_val_if_fail(pixbuf != NULL, NULL);
    g_return_val_if_fail(GDK_IS_PIXBUF(pixbuf), NULL);

    /* Returns a copy of pixbuf with it's non-completely-transparent pixels to
       have an alpha level "alpha_percent" of their original value. */

    target = gdk_pixbuf_add_alpha(pixbuf, FALSE, 0, 0, 0);

    if(alpha_percent == 1.0)
        return target;
    width = gdk_pixbuf_get_width(target);
    height = gdk_pixbuf_get_height(target);
    rowstride = gdk_pixbuf_get_rowstride(target);
    data = gdk_pixbuf_get_pixels(target);

    for(y = 0; y < height; y++) {
        for(x = 0; x < width; x++) {
            /* The "4" is the number of chars per pixel, in this case, RGBA,
               the 3 means "skip to the alpha" */
            current = data + (y * rowstride) + (x * 4) + 3;
            *(current) = (guchar)(*(current) * alpha_percent);
        }
    }

    return target;
}


static GdkPixbuf*
scale_or_ref(GdkPixbuf *src,
             int width,
             int height)
{
    if(width == gdk_pixbuf_get_width(src) &&
            height == gdk_pixbuf_get_height(src)) {
        return g_object_ref(src);
    } else {
        return gdk_pixbuf_scale_simple(src, width, height, GDK_INTERP_BILINEAR);
    }
}


static GdkPixbuf *
render_icon(GtkStyle            *style,
            const GtkIconSource *source,
            GtkTextDirection     direction,
            GtkStateType         state,
            GtkIconSize          size,
            GtkWidget           *widget,
            const char          *detail)
{
    (void) direction;
    (void) detail;
    int width = 1;
    int height = 1;
    GdkPixbuf *scaled;
    GdkPixbuf *stated;
    GdkPixbuf *base_pixbuf;
    GdkScreen *screen;
    GtkSettings *settings;

    /* Oddly, style can be NULL in this function, because
     * GtkIconSet can be used without a style and if so
     * it uses this function.
     */

    base_pixbuf = gtk_icon_source_get_pixbuf(source);

    g_return_val_if_fail(base_pixbuf != NULL, NULL);

    if(widget && gtk_widget_has_screen(widget)) {
        screen = gtk_widget_get_screen(widget);
        settings = gtk_settings_get_for_screen(screen);
    } else if(style->colormap) {
        screen = gdk_colormap_get_screen(style->colormap);
        settings = gtk_settings_get_for_screen(screen);
    } else {
        settings = gtk_settings_get_default();
        GTK_NOTE(MULTIHEAD,
                 g_warning("Using the default screen for gtk_default_render_icon()"));
    }

    if(size != (GtkIconSize) - 1 && !gtk_icon_size_lookup_for_settings(settings, size, &width, &height)) {
        g_warning(G_STRLOC ": invalid icon size '%d'", size);
        return NULL;
    }

    /* If the size was wildcarded, and we're allowed to scale, then scale; otherwise,
     * leave it alone.
     */
    if(size != (GtkIconSize) - 1 && gtk_icon_source_get_size_wildcarded(source))
        scaled = scale_or_ref(base_pixbuf, width, height);
    else
        scaled = g_object_ref(base_pixbuf);

    /* If the state was wildcarded, then generate a state. */
    if(gtk_icon_source_get_state_wildcarded(source)) {
        if(state == GTK_STATE_INSENSITIVE) {
            stated = set_transparency(scaled, 0.3);
            gdk_pixbuf_saturate_and_pixelate(stated, stated, 0.1, FALSE);

            g_object_unref(scaled);
        } else if(state == GTK_STATE_PRELIGHT) {
            stated = gdk_pixbuf_copy(scaled);

            gdk_pixbuf_saturate_and_pixelate(scaled, stated, 1.2, FALSE);

            g_object_unref(scaled);
        } else {
            stated = scaled;
        }
    } else
        stated = scaled;

    return stated;
}

static void
oxygenengine_style_class_init(OxygenEngineStyleClass *klass)
{
    GtkStyleClass *style_class = GTK_STYLE_CLASS(klass);

    parent_class = g_type_class_peek_parent(klass);

    style_class->draw_hline = draw_hline;
    style_class->draw_vline = draw_vline;
    style_class->draw_shadow = draw_shadow;
    style_class->draw_polygon = draw_polygon;
    style_class->draw_arrow = draw_arrow;
    style_class->draw_diamond = draw_diamond;
    /*style_class->draw_string = draw_string;*/
    style_class->draw_box = draw_box;
    style_class->draw_flat_box = draw_flat_box;
    style_class->draw_check = draw_check;
    style_class->draw_option = draw_option;
    style_class->draw_tab = draw_tab;
    style_class->draw_shadow_gap = draw_shadow_gap;

    /* box around notebooks */
    style_class->draw_box_gap = draw_box_gap;
    /* the tab */
    style_class->draw_extension = draw_extension;

    style_class->render_icon = render_icon;

    style_class->draw_focus = draw_focus;
    style_class->draw_handle = draw_handle;
    style_class->draw_layout = draw_layout;

    style_class->draw_slider = draw_slider;

}