/***************************************************************************
 *   Copyright (C) 2010 by Cédric Bellegarde                               *
 *   gnumdk@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QDebug>
#include "smartcache.h"
 

// Insert item
void SmartCache::insert(QSize size, QImage *image, bool scroll)
{
    Object *o = new Object(size, image, scroll);
    Object *tmp = NULL;
    QSize s;

    tmp = item(size);
    if(tmp && tmp->m_scroll == scroll) // already in cache
        return;
    else
        remove(size);

    // Make room for new item
    if(m_list.size() == m_maxItems) {
        Object *toRemove = m_list.takeAt(0);
        delete toRemove;
    }

    m_list.append(o);
}

void SmartCache::remove(QSize size)
{
    int i;
    Object *o = NULL;
    for(i = 0; i < m_list.size(); i++) {
        // delete item if not important
        if(m_list.value(i)->m_size == size) {
            o = m_list.takeAt(i);
            delete o;
            break;
        }
    }
}

Object* SmartCache::item(QSize size)
{
    int i;
    Object *o;

    for(i = 0; i < m_list.size(); i++) {
        if(m_list.value(i)->m_size == size) {
            o = m_list.value(i);
            if(!o->m_scroll)
                return o;
        }
    }
    return NULL;
}

Object* SmartCache::scrollItem(QSize size)
{
    int i;
    Object *o;

    for(i = 0; i < m_list.size(); i++) {
        if(m_list.value(i)->m_size == size) {
            o = m_list.value(i);
            if(o->m_scroll)
                return o;
        }
    }
    return NULL;
}

void SmartCache::setMaxItems(int max)
{
    m_maxItems = max;
}

void SmartCache::debug()
{
    int i;
    Object *o;
    for(i = 0; i < m_list.size(); i++) {
        o = m_list.value(i);
        qDebug() << o->m_size << o->m_scroll << o->m_image << "\n";
    }
    qDebug() << "*********************************************************\n";
}