/***************************************************************************                                                             *
 *   Copyright (C) 2010 by Cédric Bellegarde                               *
 *   gnumdk@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SMARTCACHE_H
#define SMARTCACHE_H

#include <QList>
#include <QSize>
#include <QImage>

#include <gtk/gtk.h>
#include <gtk/gtkstyle.h>
#include <gdk/gdkx.h>

class Object
{
public:
    inline Object(QSize size, QImage *image, bool scroll) {
        m_scroll = scroll;
        m_size = size;
        m_image = image;
    }
    inline ~Object() {
        if(m_image)
            delete m_image;
    }
    QSize m_size;
    bool m_scroll;
    QImage *m_image;
};

class SmartCache
{
public:
    inline SmartCache() {
        m_maxItems = 5;
    }
    // insert item
    void insert(QSize size, QImage *image, bool scroll);
    // remove item in list, free it
    void remove(QSize size);
    // return item with xid
    Object* item(QSize size);
    // return scroll item
    Object* scrollItem(QSize size);
    // set maximum items in list
    void setMaxItems(int max);
    void debug();
private:
    QList<Object*> m_list;
    int m_maxItems;
};

#endif
