/***************************************************************************
 *   Copyright (C) 2008 by David Sansome                                   *
 *   me@davidsansome.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "utilities.h"
#include "engine.h"

#ifdef USE_FREEBSD
#include <kvm.h>
#include <paths.h>
#include <sys/param.h>
#include <sys/sysctl.h>
#include <sys/user.h>
#include <sys/file.h>
#endif

#ifdef USE_SOLARIS
#include <procfs.h>
#endif

#include <QFile>
#include <QDebug>

#include <gtk/gtk.h>

QString GtkQtUtilities::colorString(const QColor& color)
{
    QString ret = "{";
    ret += QString::number(color.red() * 257) + ", ";
    ret += QString::number(color.green() * 257) + ", ";
    ret += QString::number(color.blue() * 257) + "}";

    return ret;
}

QColor GtkQtUtilities::convertColor(const GdkColor& color)
{
    int r = color.red / 257;
    int g = color.green / 257;
    int b = color.blue / 257;
    return QColor(r, g, b);
}

GdkColor GtkQtUtilities::convertColor(const QColor& color, GtkStyle* style)
{
    GdkColor ret;
    ret.red = color.red() * 257;
    ret.green = color.green() * 257;
    ret.blue = color.blue() * 257;

    gdk_colormap_alloc_color(style->colormap, &ret, false, true);

    return ret;
}


void GtkQtUtilities::parseRcString(const QString& string)
{
    gtk_rc_parse_string(string.toAscii().constData());
    GTK_QT_DEBUG("Setting RC string:" << string.trimmed())
}

void GtkQtUtilities::parseRcString(const QString& defs, const QString& pattern, StyleType type)
{
    parseRcString(generateRcString(defs, pattern, type));
}

// Thanks Martin Dvorak of metatheme
QString GtkQtUtilities::generateRcString(const QString& defs, const QString& pattern, StyleType type)
{
    static int count = 0;
    count++;

    QString typeStr;
    switch(type) {
    case Class:       typeStr = "class";        break;
    case WidgetClass: typeStr = "widget_class"; break;
    case Widget:      typeStr = "widget";       break;
    }

    QString countStr(QString::number(count));
    QString ret("style \"gtk-qt-dynamic-" + countStr + "\" { " + defs + " } " + typeStr + " \"" + pattern + "\" style \"gtk-qt-dynamic-" + countStr + "\"\n");

    return ret;
}
