/***************************************************************************
 *   Copyright (C) 2008 by David Sansome                                   *
 *   me@davidsansome.com                                                   *
 *                                                                         *
 *   Copyright (C) 2010 by Cédric Bellegarde                               *
 *   gnumdk@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "colormapper.h"
#include "engine.h"
#include "utilities.h"
#include "rcproperties.h"

#include <QColor>
#include <QMap>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "wrapper.h"
#include "qt_style.h"

#define GTK_QT_SETUP_STATE \
    if (!s_engine->isEnabled() || !s_engine->setRect(x, y, w, h)) \
    { \
        return; \
    } \
    s_engine->setWindow(window, style, state, toplevel); \
    s_engine->setHasFocus(hasFocus); \
     
Engine* s_engine;

static QMap<unsigned long, QSize> windowSizes;

void gtkQtInit()
{
    s_engine = Engine::instance();
}

void gtkQtDestroy()
{
    delete s_engine;
    s_engine = NULL;
}

int gtkQtDebug()
{
    return Engine::instance()->isDebug() ? 1 : 0;
}

gboolean isStyle(const char* name)
{
    return ! strcmp(s_engine->styleName(), name);
}

void addSize2Map(unsigned long xid, int width, int height)
{
    windowSizes.insert(xid, QSize(width, height));
}

void delSizeFromMap(unsigned long xid)
{
    windowSizes.remove(xid);
}

void sizeFromMap(unsigned long xid, int *width, int *height)
{
    QSize size = windowSizes.value(xid);
    *width = size.width();
    *height = size.height();
}

void drawButton(GTK_QT_WRAPPER_ARGS, gboolean defaultButton)
{
    GTK_QT_SETUP_STATE
    s_engine->drawButton(defaultButton);
}

void drawToolButton(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawToolButton();
}

void drawTab(GTK_QT_WRAPPER_ARGS, GtkPositionType position)
{
    QTabWidget::TabPosition pos;

    switch(position) {
    case GTK_POS_LEFT:
        pos = QTabWidget::West;
        break;
    case GTK_POS_RIGHT:
        pos = QTabWidget::East;
        break;
    case GTK_POS_BOTTOM:
        pos = QTabWidget::South;
        break;
    default:
        pos = QTabWidget::North;
    }
    GTK_QT_SETUP_STATE
    s_engine->drawTab(pos);
}

void drawLineEdit(GTK_QT_WRAPPER_ARGS)
{
    // Lineedit drawing is split into two functions.  drawLineEditFrame is called first with the whole area, then drawLineEdit
    // is called with just the interior.  It is also clipped to the interior area so we can't overdraw.
    x -= style->xthickness;
    y -= style->ythickness;
    w += style->xthickness * 2;
    h += style->ythickness * 2;

    GTK_QT_SETUP_STATE
    s_engine->drawLineEdit();
}

void drawLineEditFrame(GTK_QT_WRAPPER_ARGS, int tx, int ty, gboolean isMozilla)
{
    GTK_QT_SETUP_STATE
    s_engine->drawLineEditFrame(tx, ty, isMozilla);
}

void drawFrame(GTK_QT_WRAPPER_ARGS, int type, int tx, int ty, GtkPositionType position)
{
    GTK_QT_SETUP_STATE
    QTabWidget::TabPosition pos;

    switch(position) {
    case GTK_POS_LEFT:
        pos = QTabWidget::West;
        break;
    case GTK_POS_RIGHT:
        pos = QTabWidget::East;
        break;
    case GTK_POS_BOTTOM:
        pos = QTabWidget::South;
        break;
    default:
        pos = QTabWidget::North;
    }
    s_engine->drawFrame(type, tx, ty, pos);
}

void drawComboBox(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawComboBox();
}

void drawRectFrame(GTK_QT_WRAPPER_ARGS, int tx, int ty)
{
    GTK_QT_SETUP_STATE
    s_engine->drawRectFrame(tx, ty);
}

void drawCheckBox(GTK_QT_WRAPPER_ARGS, gboolean checked)
{
    GTK_QT_SETUP_STATE
    s_engine->drawCheckBox(checked);
}

void drawMenuCheck(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawMenuCheck();
}

void drawRadioButton(GTK_QT_WRAPPER_ARGS, gboolean checked)
{
    GTK_QT_SETUP_STATE
    s_engine->drawRadioButton(checked);
}


void drawScrollBarSlider(GTK_QT_WRAPPER_ARGS, GtkOrientation orientation)
{
    if(orientation == GTK_ORIENTATION_HORIZONTAL) {
        y -= 1;
        h += 2;
        x -= RcProperties::scrollBarButtonSize() + 1;
        w += RcProperties::scrollBarButtonCount() * RcProperties::scrollBarButtonSize() + 2;
    } else {
        x -= 1;
        w += 2;
        h += RcProperties::scrollBarButtonCount() * RcProperties::scrollBarButtonSize() + 2;
        y -= RcProperties::scrollBarButtonSize() + 1;
    }

    GTK_QT_SETUP_STATE
    s_engine->drawScrollBarSlider(orientation);
}


void drawScrollBar(GTK_QT_WRAPPER_ARGS, GtkOrientation orientation)
{
    GTK_QT_SETUP_STATE
    s_engine->drawScrollBar(orientation);
}

void drawMenuBarItem(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawMenuBarItem();
}

void drawToolMenuItem(GTK_QT_WRAPPER_ARGS, int type, gboolean toolbar)
{
    GTK_QT_SETUP_STATE
    s_engine->drawToolMenuItem(type, toolbar);
}

void drawSplitter(GTK_QT_WRAPPER_ARGS, GtkOrientation orientation)
{
    GTK_QT_SETUP_STATE
    s_engine->drawSplitter(orientation);
}

/*
void drawTabFrame(GTK_QT_WRAPPER_ARGS, unsigned long xid)
{
    if(!s_engine->isEnabled())
        return;

    int overlap = s_engine->style()->pixelMetric(QStyle::PM_TabBarBaseOverlap);
    y -= overlap;
    h += overlap;

    GTK_QT_SETUP_STATE
    s_engine->drawTabFrame(xid);
}
*/

void drawMenuFrame(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawMenuFrame();
}

void drawProgressChunk(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawProgressChunk();
}

void drawProgressBar(GTK_QT_WRAPPER_ARGS, GtkProgressBarOrientation orientation, double percentage)
{
    GTK_QT_SETUP_STATE
    s_engine->drawProgressBar(orientation, percentage);
}

void drawSlider(GTK_QT_WRAPPER_ARGS, GtkAdjustment* adj, GtkOrientation orientation, int inverted)
{
    GTK_QT_SETUP_STATE
    s_engine->drawSlider(adj, orientation, inverted);
}

void drawSpinButton(GTK_QT_WRAPPER_ARGS, int direction)
{
    GTK_QT_SETUP_STATE
    s_engine->drawSpinButton(direction);
}

void drawSpinFrame(GTK_QT_WRAPPER_ARGS, int tx, int ty)
{
    GTK_QT_SETUP_STATE
    s_engine->drawSpinFrame(tx, ty);
}

void drawListHeader(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawListHeader();
}

void drawSquareButton(GTK_QT_WRAPPER_ARGS)
{
    (void) window;
    (void) toplevel;
    (void) style;
    (void) state;
    (void) x;
    (void) y;
    (void) w;
    (void) h;
    (void) hasFocus;
}

void drawArrow(GTK_QT_WRAPPER_ARGS, GtkArrowType direction)
{
    GTK_QT_SETUP_STATE
    s_engine->drawArrow(direction);
}

void drawTooltip(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawTooltip();
}

void drawItemViewItem(GTK_QT_WRAPPER_ARGS, unsigned int round)
{
    GTK_QT_SETUP_STATE
    s_engine->drawItemViewItem(round);
}

void drawRect(GTK_QT_WRAPPER_ARGS, gboolean fill)
{
    GTK_QT_SETUP_STATE
    s_engine->drawRect(fill);
}

void drawOxygenBackground(GTK_QT_WRAPPER_ARGS)
{
    GTK_QT_SETUP_STATE
    s_engine->drawOxygenBackground();
}

void drawWidgetBackground(GTK_QT_WRAPPER_ARGS, int tx, int ty)
{
    GTK_QT_SETUP_STATE
    s_engine->drawWidgetBackground(tx, ty);
}

void drawViewportBackground(GTK_QT_WRAPPER_ARGS, int tx, int ty)
{
    GTK_QT_SETUP_STATE
    s_engine->drawViewportBackground(tx, ty);
}

GdkColor alternateBackground(GtkStyle* style, gboolean enabled)
{
    QColor qcolor = QApplication::palette().color(enabled ? QPalette::Active : QPalette::Disabled, QPalette::AlternateBase);
    return GtkQtUtilities::convertColor(qcolor, style);
}

void destroyImage(unsigned long xid, int width, int height)
{
    delSizeFromMap(xid);
    s_engine->destroyImage(QSize(width, height));
}

gboolean noGradients()
{
    return s_engine->noGradients();
}